( Sound Generation for 3-instruction Forth on Atmega328P uC / Arduino Nano board )
\ Assad Ebrahim, assadebrahim2000@gmail.com, http://www.mathscitech.org/articles
\ 2020-06-22, 2020-07-25 edited, 2021-05-16---Sun---01:18

( * WORDS FOR COMPUTERIZED MUSIC * )

\ Hardware Options:
\   Buzzer: 5V,  + on pin, - on ground, use tone 440 --- loud sound
\   8 OHM Speaker with 250R or greater resistor.  resistor between pin and speaker+, speaker - to ground --- small sound
\   8 OHM Speaker with 2N2222 transistor for amplification.  
\       Wiring: 5V to collector, uC audio pin to base, spkr+ to emitter, spkr- to GRD with 100R resistor and 10uF capacitor in parallel in between-- big sound
\   
\ Firmware: Piano.avr (for 13-key direct keyboard)
\           Piano-4x4.avr (for shift-in register 8-key keyboard)

\ Design: Music is a time series of NOTES/TONES (frequencies), their DURATIONS (ms), and the RESTS (ms) between them.
\ Design: Simplest tone generation specifies frequency and uses fixed duration and fixed rest, but this is not expressive.
\ Design: Better is 3 arrays holding note frequencies, note durations, and rests.  Play reads all three to form the instructions.
\ Design: Tone frequency is selected from a frequency table of 96 semitones (0-95), spanning X octaves.
\           freq takes index and returns the frequency.  Tone takes the frequency and produces the sound.
\           33 tone   or 0 freq tone
\           note = freq tone
\           ,,  ( for simplified 1..8 on selected scale )

\ Design: Could have two speakers and voices..., for now just one
variable SPEAKERpin  
: set-speaker ( pin -- ) SPEAKERpin ! ;   \ ex: 13 set-speaker
: speaker@    ( -- pin ) SPEAKERpin @ ;

\ Design: Duration and rest are read from variables.
variable dur_ms 
: beat dur_ms @ ; 
: setbeat ( n -- ) dur_ms ! ;    
: R1 beat 3 / ms ; \ semiquaver rest (1/16 note) \ //
: R2 beat 2 * ms ; \ crotchet rest (1/4 note) \ ///
: R3 beat 4 * ms ;
: R4 beat 6 * ms ;
120 setbeat \ baseline tone duration ms  (sixteenth note - semiquaver)

\ Goal: Music tuning is a fascinating subject.  The programmable digital piano allows the exploration of this subject
\   Just (ratio) tuning:  https://en.wikipedia.org/wiki/Just_intonation

\ Design: Musical Tones 96 semitones (half steps) spanning 8 octaves (=12*8)
\ The mathematics of western music is multiplicative (equitempered scale popularized by Bach in the 1700s)
\ The musical scale has 12 semitones with each octave being double the frequency of the previous
\ By convention, there are 2 flats (b%, e%) and three sharps (c#, f#, g#)
\ Ratio between successive musical semitones is fixed as 12th-root of 2 = 2^(1/12) = 1.05946309436 (approx 6% higher in freq than last semitone)
\ Design: fixed freq table 384 bytes (=96*4) avoids cumulative tuning error vs fixed point multipl (1%, 5%, 15%, 40% for 1,2,3,4 octaves)
\ Design: For Arduino Atmega328P - It is not possible to generate tones lower than 31Hz (B0)

create piano   \ each cell in lookup table is 4 bytes
        \       0            2             4      5             7             9            11      
        \             1             3                    6             8             10
        \       C     d%     D      e%     E      F      F#     G      a%     A      b%     B
\ ( -1 )      16 ,   17 ,   18 ,   19 ,   21 ,   22 ,   23 ,   25 ,   26 ,   28 ,   29 ,   31 ,  \ 108 tones, except this too low
\ 96 piano notes (= 8 octaves x 12 semitones per octave), 1-indexed
\ Frequency chart: http://www.phys.unsw.edu.au/jw/notes.html with MIDI notes.  
\ Design: adding a null-freq to the start of the table so can also have a null-note which, went sent, sounds nothing, essentially a rest note
\ Design: To convert this table to MIDI notes, use an offset of 25 (C0)
              0  ,  \ null-freq or rest-freq
   ( 0 )      33 ,   35 ,   37 ,   39 ,   41 ,   44 ,   46 ,   49 ,   52 ,   55 ,   58 ,   62 ,  \ bass guitar lower E___
   ( 1 )      65 ,   69 ,   73 ,   78 ,   82 ,   87 ,   93 ,   98 ,  104 ,  110 ,  117 ,  124 ,  \ 6-string guitar low E__
   ( 2 )     131 ,  139 ,  147 ,  156 ,  165 ,  175 ,  185 ,  196 ,  208 ,  220 ,  233 ,  247 ,
   ( 3 )     262 ,  277 ,  294 ,  311 ,  330 ,  349 ,  370 ,  392 ,  415 ,  440 ,  466 ,  494 ,  \ middle C_
   ( 4 )     523 ,  554 ,  587 ,  622 ,  659 ,  698 ,  740 ,  784 ,  831 ,  880 ,  932 ,  988 , 
   ( 5 )    1047 , 1109 , 1175 , 1245 , 1319 , 1397 , 1480 , 1568 , 1661 , 1760 , 1865 , 1976 , 
   ( 6 )    2093 , 2217 , 2349 , 2489 , 2637 , 2794 , 2960 , 3126 , 3322 , 3520 , 3729 , 3951 , 
   ( 7 )    4186 , 4435 , 4699 , 4978 , 5274 , 5588 , 5920 , 6272 , 6645 , 7040 , 7459 , 7908 ,
\ Design: Chinese scales (pentatonic) use different frequencies from Western frequencies
\ Design: Frequencies for Chinese scales are written directly into the frequency table
\ Design: Root note pointers
\ 97-128  Chinese musical frequencies
   ( 8 yu )    440 , 529  , 594  , 660  , 793  , 440 2 * , 529 2 * , 594  2 *   ,      \ Yu scale on Huang Zhong base freq
\               A    C'+6   D'+7   E'+1   G'+9   A'        C''+11    D''+13
   ( 9 shang ) 495 , 557  , 669  , 743  , 892  , 495 2 * , 557 2 * , 669  2 *   ,      \ Shang scale on Da Cu  base freq ( Snoopy theme)
\              B+1   c#'+3  e'+10  f#'+3  a'+12  b''+2     c#''+5    e''+19
  ( 10 gong  ) 594 , 669  , 752  , 895  , 1003 , 594 2 * , 669 2 * , 752  2 *   ,
\              D'+7  e'+10  g'+12  a'+15  b'+15  D''+13    E''+19    G''+23  
  ( 11 jue )   705 , 847  , 952  , 1129 , 1270 , 705 2 * , 847 2 * , 952  2 *   ,      \ Jue scale on Yi Ze base freq 
  ( 12 zhi )   793 , 892  , 1071 , 1189 , 1338 , 793 2 * , 892 2 * , 1071 2 *  ,      \ Zhi scale on Wu Yi base freq


\ MICROTONAL UNITS - Arabic & N. African, and Indian music 
\ Ratio between successive musical quartertones is fixed as 24th-root of 2 = 2^(1/24) = 1.0293 (approx 3% higher in freq than last tone)
\ Maths behind it: https://www.myriad-online.com/resources/docs/harmony/english/microtone.htm
\ Quarter tone theory: https://en.wikipedia.org/wiki/Quarter_tone
\ Arab tone system: https://en.wikipedia.org/wiki/Arab_tone_system
\ Quartertone Frequency reference chart: https://www.academia.edu/28966442/Quarter_tone_frequencies
\ Also:  https://www.reddit.com/r/musictheory/comments/154sdn/does_anyone_know_where_i_can_find_a_frequency_in/c7jinxq?utm_source=share&utm_medium=web2x&context=3
\ Octave in W. music - 24 quartertones
\ Indian srutis - 22 notes per octave - https://www.britannica.com/art/sruti

\ index into frequency table for Chinese scales
97  constant yu-root  \ yu scale on Huang Zhong base freq (why don't you come back)
105 constant shang-root \ shang scale on Da Cu base freq (back to life)
113 constant gong-root
121 constant jue-root
129 constant zhi-root
\ 137


\ Design: keeping sound and lights separate in fundamental words.  See joint interface in 3iSonglist.fs
\ Design: interface for 1..8 notation is in 3iSonglist.fs

: freq ( note-index -- ) cells piano + @ ;  \ ex: 36 freq tone = C_ (middle C)
: tone ( freq -- )       lohi beat lohi speaker@ [tone] R2 ;  \  ex: 440 tone  or 45 freq tone
: note ( note-index -- ) freq tone ;
: f2i  ( freq -- index )  
   >r piano begin @+ r@ = until      \ search array for freq 
   piano -  4 bytes / 1 -  r> drop ; \ calculate index


\ =============== READING KEYBOARD PRESSES from serial
\ Design: to respond to keyboard presses need uC to send serial notifications when a key is pressed, turn on with 1 buttonsrecord (from 3iForth.fs)
\ Design: key press notifications are 6-byte tuples  (from 3iRecord.fs)
: fetch-note-data  ( <serial> -- numbytes ) 6 read-com ;  \ reads from serial buffer into rx-buffer, indicates if data is present (for polling use)
: show-note-data   ( -- ) rx-buffer 6 show-rx 2drop ;
: [parse-note] ( adr -- adr' val16 )  \ reads byte buffer parsed as little-endian words with low byte first to val16
    c@+ swap      c@+ 8 lshift     rot    or ;
: extract-note ( -- 1-8# or 10-14# ) rx-buffer 2 wwords + [parse-note] nip ;
: poll-note-data   ( <serial> -- note# ) begin fetch-note-data 0 > if  extract-note exitloop else continueloop then until ;
: >// 200 ms ;


\ ==================  NOTES
\ This is 1-indexed skipping over null note
\ Design: # and % decorates denote sharp/flat and appear after note name
\ Design: _ and ' decorations denote lower and upper octaves on given note.  Middle C octave is _
\ Usage: c_ freq tone  or c_ note
\ Desgin: null-freq / null-note assigned to keys make no noise
0   constant  null-freq
0   constant  null-note
1   constant  C____   \ octave lower 3
2   constant  c#____
2   constant  d%____ 
3   constant  D____ 
4   constant  d#____
4   constant  e%____
5   constant  E____ 
6   constant  F____ 
7   constant  f#____
7   constant  g%____
8   constant  G____      
9   constant  g#____
9   constant  a%____
10   constant  A____ 
11  constant  a#____
11  constant  b%____
12  constant  B____ 
13  constant  C___   \ octave lower 2
14  constant  c#___
14  constant  d%___ 
15  constant  D___ 
16  constant  d#___
16  constant  e%___
17  constant  E___ 
18  constant  F___ 
19  constant  f#___
19  constant  g%___
20  constant  G___      
21  constant  g#___
21  constant  a%___
22  constant  A___ 
23  constant  a#___
23  constant  b%___
24  constant  B___ 
25  constant  C__    \ octave lower 1
26  constant  c#__ 
26  constant  d%__ 
27  constant  D__  
28  constant  d#__ 
28  constant  e%__ 
29  constant  E__  
30  constant  F__    \  START of 32-key keyboard from F below middle C
31  constant  f#__ 
31  constant  g%__ 
32  constant  G__       
33  constant  g#__ 
33  constant  a%__  
34  constant  A__         
35  constant  a#__ 
35  constant  b%__ 
36  constant  B__  
37  constant  C_    \ middle octave -- Middle C
38  constant  c#_  
38  constant  d%_  
39  constant  D_    
40  constant  d#_  
40  constant  e%_  
41  constant  E_    
42  constant  F_    
43  constant  f#_  
43  constant  g%_   
44  constant  G_   
45  constant  g#_      
45  constant  a%_  
46  constant  A_   
47  constant  a#_  
47  constant  b%_   
48  constant  B_   
49  constant  C'   \ octave upper 1
50  constant  c#'  
50  constant  d%'  
51  constant  D'   
52  constant  d#'  
52  constant  e%'  
53  constant  E'   
54  constant  F'   
55  constant  f#'  
55  constant  g%'  
56  constant  G'   
57  constant  g#'  
57  constant  a%'  
58  constant  A'   
59  constant  a#'  
59  constant  b%'  
60  constant  B'   
61  constant  C''    \ octave upper 2  --- END Of 32 key keyboards
62  constant  c#'' 
62  constant  d%'' 
63  constant  D''  
64  constant  d#'' 
64  constant  e%'' 
65  constant  E''  
66  constant  F''  
67  constant  f#'' 
67  constant  g%'' 
68  constant  G''  
69  constant  g#'' 
69  constant  a%'' 
70  constant  A''  
71  constant  a#'' 
71  constant  b%'' 
72  constant  B''  
73  constant  C'''   \ octave upper 3
74  constant  c#'''
74  constant  d%'''
75  constant  D''' 
76  constant  d#'''
76  constant  e%''' 
77  constant  E''' 
78  constant  F''' 
79  constant  f#'''
79  constant  g%'''
80  constant  G''' 
81  constant  g#'''
81  constant  a%'''
82  constant  A''' 
83  constant  a#'''
83  constant  b%'''
84  constant  B'''   \ 12-bits of frequency goes up to B'''
85  constant  C''''  \ octave upper 4

\ =================  SOUND DEMO ROUTINES and Effects ====================================
96 constant semitones  \ spanning 8 octaves
: upward  semitones 0 do i 1 + note loop ;
: downward semitones 0 do semitones i - note loop ;
: updown upward downward ;
: updown-fast 60 setbeat updown 120 setbeat ;
: updown-sfast 30 setbeat updown 120 setbeat ;
: updown-slide 15 setbeat updown 120 setbeat ;
: ring ( -- ) a_ freq lohi 500 lohi speaker@ [tone] ; 
: ring-ring  ( -- )  ring 600 ms ring ; \ ring tone


\ =================  APPLICATIONS =====================
\ 3iArpegiattor.fs : relies on board returning which key was pressed (turn on record/respond feature)
\ 3iPianoLights.fs : for 32 light board arranged in piano key layout (white/black keys) starting at F__ as 1 and up to C''



\ Music theory:
\ Key signature: https://en.wikipedia.org/wiki/Key_signature
\   There are fifteen homogeneous key signatures ranging from 7 flats to 7 naturals to 7 sharps, with each successive flat or sharp placed on the note a perfect fifth below or above the previous one
\ Keys Major and Minor: https://en.wikipedia.org/wiki/Major_and_minor#Keys
\ In Western music, there are 12 semitones (half steps).  A scale picks out 8 of these.
\ A major scale has a HALFSTEP between notes 3 & 4 and between notes 7 & 8  (eg C major between E-F and B-C)
\ A minor scale puts the HALFSTEP between notes 2 & 3 and between notes 5 & 6 (eg D minor between E-F and A-B%)
\ Octave is a 2:1 ratio, Fifth is a 3:2 ratio
\ Intonation: 12-tone equal temperament is the most common tuning system in the West


\ * UNIVERSAL 8-KEY INSTRUMENT - NUMBERS TO NOTES *
\ Design: data structure for storing scale is 14 notes, first the 8 notes of the diatonic scale, and then the 5 notes of (possible) chromatic scale, or null
\ V1 Design: diatonic scale is an octet (8-tuple) of notes (piano index positions) that sound good together
\ V2 (idea) Design: A scale (key signature)is a root (note) plus K semitone intervals (+/-) forming a scale of N = K+1 notes.  Ex:  C_ (0) + {2,4,5,7,9,11,12}
\ Design: scales simplify tunes to 1..8 instead of requiring 13 chromatic notes, enabling a programmable/universal piano with fewer keys to play it
\ Design: raising/lowering octave and transposing by semitones makes a limited key piano a universal instrument

\ LOCAL (VIRTUAL) SCALE: Setting up Key Signatures / Scales, and playing by reference against a scale

\ V2 Design: a std-scale has 14 notes: 8 white keys (7+1 root at higher octave) and 6 black keys (5+1 root at higher octave or 6 null-freq)
variable scale-ptr
: scale      ( -- a* )  scale-ptr @ ;
: set-scale  ( a* -- )  scale-ptr ! ;
\ Design:  For scales provided as a list on the stack, use tmp14 for memory (as transp works from memory)
create tmp14 14 bytes allot 
: 5nulls  0 0 0 0 0 ;
: show-scale  ( scaleptr -- ) 14 .mem ;
\ Design: shows the relative shifts that make up the first 8 diatonic notes of the scale
: scale-delta ( scaleptr -- incrlist ) dup c@ swap   8 0 do c@+ swap -rot over - -rot swap loop 2drop ;  \ ex: d%maj8 show-scale   cstd8 show-scale

\ setup frequency-key mappings on target piano to enable changing scale/key signature/ethnic piano
create scale-registers 13 bytes allot  \ on uC
: white-registers   0 2 4 5 7 9 11 12 ;
: black-registers   1 3 6 8 10  ; 
white-registers black-registers 13 scale-registers >mem drop

: assign-key       ( freq keypos0-12 -- ) scale-registers + c@ setreg ;
: assign-black-key ( freq keypos0-4 -- ) 8 + assign-key ;
: black-null       ( -- ) 5 0 do null-freq i assign-black-key loop ;
\ CODE-REVIEW: always transmits 13 (matching piano regs in uC), more than that and need to adjust the piano register memory perhaps to 24 contiguous slots...
: xscale ( scaleptr -- )  13 0 do c@+ freq i assign-key loop drop ;  \ send 14-note scale in memory to uC 
\ SCALE MANAGEMENT
\ Design: sends list (maxlen 14) of scale notes on stack to piano uC (target), white keys first, then black keys, null for all that are not specified
: fill-nulls ( nlen<14 -- <xx> nlen' )  \ adds 14 - nlen zeros to stack...
    dup 14 - dup 0 < if nip negate 0 do null-note loop 14 else drop then ;
\ Design: also writes to tmp14 memory
: >xscale ( noteslist nlen -- ) fill-nulls tmp14 >mem  tmp14 xscale ;


\ TRANSPOSITION BY SEMITONES
\ Design: semi# holds number of semitones to transpose by.  Positive and negative?
\ Design: semi# works on notes in memory e.g. 36 or c_ -- see songlist, or uses tmp14 for scales stored to get to uC
variable semi#
: set-semi#       ( semitones -- ) semi# c! ;
: get-semi#       ( semitones -- ) semi# c@ ;

: [,,]       ( 1-8# or 9-14# -- note )  1 - scale + c@ ;  \ 1-indexed
:  ,,        ( 1-8# or 9-14# -- )       [,,] note ;        
: play-scale ( -- ) 9 1 do i ,, loop ;

\ Design: transposeN works on N (1-14) notes, but if it's null-freq, keeps it null freq
\ Design: xscaleN pushes N (1-14) notes from scale to the uC - allows for support of variable sized keyboards 2,3,5,8,13,24,25,32
\ Design: ? list or memory ?  probably best for memory, and use >mem and mem> for going between list and mem
\ Design: stack is the input interface for user-provided scales because it is user-friendly
\ CODE REVIEW: check robustness of load-scale not requiring nlen...
: save-scale ( <create cstd14> <notelist1..14> nlen<14 -- nlen )  \ writes N notes to memory (generated on fly)
    here >r fill-nulls dup bytes allot r> >mem drop ;  
\ Design: For scales using non-western frequencies, e.g. Chinese scales, we write the new frequencies directly at the end of the frequency table
: save-exotic-scale ( <create keyname> root-index -- ) 8 0 do dup i + swap loop drop 8 save-scale ;
: transpose-scale   ( scaleptr semi# -- scaleptr' )  \ transposes scale into tmp14 location, null-note are retained as null-note
    set-semi# tmp14 14 0 do 
        swap c@+ dup 
        null-note = if ( no change ) 
                    else get-semi# + 
                    then rot c!+ 
    loop 2drop tmp14 ;
: load-scale ( scaleptr -- ) dup set-scale xscale ;
: show-scale ( scaleptr -- ) 8 swap .mem drop ;
\ Design: instead of transposing scales, one usually cycles them, lowering the octave of the right-most notes
\ Design: but some scales of 8 are pentatonic then repeat, so the cycle needs the uniq pattern length
\ Design: rotates the first N main notes (white keys) - for a non-exotic scle
\                   yu8-approx: A_ C' D' E' G' A' C'' D''
\                   wanted:     G_ A_ C' D' E' G' A' C''
\                   command:  yu8-approx 5 1 cycle-scale 
\                   result:   sets up for fire on the mountain starting on key#2

: cycle-scale ( scaleptr Nuniq cycle# -- scaleptr' )  \ e.g. for a pentatonic scale like yu8-approx, Nuniq=5, cycle=2
  3 ndup nrev drop >r >r >r
  \ get Nth notes, rshift# of them, lower by an octave, and write them
  swap 1 - bytes rot + swap 0 do c@- 12 - swap loop drop r> nrev
  \ get rest of notes up to the 8 note scale
  8 swap - r> drop r> swap 0 do c@+ swap loop drop
  \ fill-nulls, and store
  8 fill-nulls tmp14 >mem drop tmp14 load-scale ;


\ Cstd8 is Bach's equitemper scale, do-re-me tuning
\ DESIGN: the 5 black keys are appended onto the 8 white keys
create cstd14  c_ d_ e_ f_ g_ a_ b_ c'    c#_ d#_ f#_ g#_ a#_ c#'  14  save-scale    \ white keys followed by black keys extended to c#' to allow [,,,] to work
: cstd cstd14    load-scale   ;

\ Design: programmatically set all keys (white and black) to lower or higher octave
: ><< ( -- ) scale -12 transpose-scale load-scale ;
: >>> ( -- ) scale  12 transpose-scale load-scale ;


\ **************************
include 3iMusic-scales.fs


\ ======== SETUP


\ Design: not needed since Piano.avr starts up with cstd scale
: init-keys  \ this is now in Setup code of Piano.avr
   10 13 setreg  \ set min duration (ms) of note
   5 14 setreg   \ set settle time following note
;

\ Design: start here
: start-speaker ( pin -- )  \ programmable digital synthesizer
    dup out
    set-speaker
	cstd play-scale  ;    \ nice chord: b.. d. g. 

\  : start-piano4x4  ( pin -- ) \ programmable digital synthesizer PCB with lights board 
\       start-speaker
\       out4 use-port-out  ;




: help-misc
    cr ."           440 tone      - sends 440 Hz (A) note to speaker"
    cr ."    c. d. e. c' c'' c.. c... f'# a.% -  sends tone to pin 13 (speaker@)  c. is middle, c.. is octave lower, c' is octave higher, # sharp, % flat"
    cr ."    1 .. 2 ..  - convert numbered simple piano to notes"
    cr
    cr ."  playlist   - runs through wings, harp, ode, memory, daffodils, morning, stairway, freedom, rainbow, reason, scarborough "
    cr ."  compositions: Assad: yearning snake  Jasmine: tog (together) "
    cr ."  => SOUND / MUSIC (tone)   include 3iSounds.fs"  
    cr ."       13 start-speaker  - sets pin 13 to speaker" 
    cr ."       record         - toggles recording mode for the keys (turns digital piano serial transmit on/off)"
    cr ."       save-bar       - saves 5 notes"
    cr ."       play-notes     - plays all saved notes"
    cr ."       delete-note    - delete last saved note"
;

: help-sound
    cr
    cr ." HELP-SOUND"
    cr ." ----------"
    cr
    cr ." Get started:"
    cr ."     plug in digital piano"
    cr ."     use RealTerm to open the connection to the port"
    cr ." In GForth:"
    cr ."     include 3iForth.fs   include 3iPort4x4.fs   include 3iSound4x4.fs"
    cr ."     1 open-com  13 start-speaker   or   13  start-piano4x4"
    cr ." Several ways to play:"
    cr ."    1.  440 tone  - play  frequency 440Hz A"
    cr ."    2.  45 note - play 45th note (440Hz A) from zero point, organized in octaves of 12 notes each"
    cr ."    3.  c_ note   c__ note   c' note   c'' note    c#__ note  b%__  note  - plays note, any octave, sharp or flat"
    cr ."    4.  1 ,,  2 ,,  - plays the diatonic (white key) notes from current scale (8 preferred keys)"
    cr ." Programming scales/key-signatures:"
    cr ."    5.  ><<   >>>  - transpose keyboard by an octave up or down "
    cr ."    6.  cstd  1flat 2flats 3flats 4flats 5flats 1sharp 2sharps 3sharps 4sharps 5sharps - Western scales"
    cr ."    7.  bhairav  yu  
    cr ."    7.  play-scale   "
    cr ."    9.  cstd  1 2 3  1 3 1 3  7 play-bar - plays n notes from stack"
    cr
    cr ."   10.  cstd8 show-scale"
    cr ."   11.  create name8 ... save-scale     name8 load-scale"
    cr ."   12.  1 buttonsrecord  (turns on serial notification of button press, see with rx)"
    cr ." More help:  help-scales  help-misc  help-record   help-playlist"
;


\ ****************************
help-sound


\ =======================================
\ Version History
\ V2 - 2020-10-03---Sat---22:00 - Expanded to include the lights capability with Piano-4x4
\ V3 - 2021-05-16---Sun---17:14 - perfected and simplified the multi-octave notation with the piano-lights board 32-keys, playlist works with play that also teaches with lights
\ V4 - 2021-08-13---Fri---04:37 - cleaning up scales/key signatures design to make it much simpler to create with Indian raga scales --- RESULT: an 8-key concept with 4 buttons
\ V5 - 2021-08-17---Tue---21:23 - encoded Chinese scales directly into frequency tables
\ V6 - 2021-09-04---Sat---02:46 - major redesign, simplifying scales using 14 notes concatenated white first then black, allowing full transposition of the entire keyboard, without nullifying blacks automatically.  Added null-freq to the piano-notes so now can nullify any key easily --- file partially complete

\ IDEA:  string instrument tuner..
\ IDEA:  song prompter for guitar...
\ http://mathscitech.org/music.html
